#!/usr/bin/env python3
# Copyright (C) 2024 Spearhead Systems SRL

import json
from datetime import datetime, timezone
from cmk.base.plugins.agent_based.agent_based_api.v1 import register, Result, Service, State


# Convert JSON entries into dictionaries indexed by name. We're assuming here
# that the name is unique across AZs and resource groups. If not, add the
# 'location' and 'resource_group' fields in each object to the name.
def parse(string_table):
    lookup = {}

    for json_str in string_table:
        obj = json.loads(json_str)
        name = obj["name"]
        group = obj["resource_group"]
        lookup[f"{name}#{resource_group}"] = obj

    return lookup


# Produce a list of Azure objects for discovery.
def discover(section):
    for name, details in sorted(section.items()):
        yield Service(item=name)


# Given a specific metric, look it up in the parsed output, and produce
# results on that service based upon the metric's range.
def check_keyvault(item, params, section):
    warn_days = params.get("warn_days")
    crit_days = params.get("crit_days")

    cert = section.get(item)
    if cert is None:
        return

    expires = datetime.fromisoformat(cert["attributes"]["expires"])
    now = datetime.now(timezone.utc)
    remaining_days = (expires - now).days

    state = State.OK
    if crit_days is not None and remaining_days < crit_days:
        state = State.CRIT
    elif warn_days is not None and remaining_days < warn_days:
        state = State.WARN

    yield Result(state=state, summary="Expires in %d days" % remaining_days)


register.agent_section(
    name="azure_keyvault_metrics",
    parse_function=parse
)

register.check_plugin(
    name="azure_keyvault_metrics",
    service_name="Azure Keyvault Metric %s",

    check_function=check_keyvault,
    check_default_parameters={},
    check_ruleset_name="azure_keyvault_metrics",

    discovery_function=discover,
)

register.agent_section(
    name="azure_firewall_metrics",
    parse_function=parse
)

register.check_plugin(
    name="azure_firewall_metrics",
    service_name="Azure Firewall Metric %s",

    check_function=check_keyvault,
    check_default_parameters={},
    check_ruleset_name="azure_firewall_metrics",

    discovery_function=discover,
)

register.agent_section(
    name="azure_defender_alerts",
    parse_function=parse
)

register.check_plugin(
    name="azure_defender_alerts",
    service_name="Azure Defender Alert %s",

    check_function=check_keyvault,
    check_default_parameters={},
    check_ruleset_name="azure_defender_alerts",

    discovery_function=discover,
)
